package gov.usgs.earthquake.nshmp.eq.model;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;

import gov.usgs.earthquake.nshmp.mfd.Mfd;

/*
 * MFD data handler class. Stores default data and creates copies with
 * overridden (non-default) fields. This class ensures that all required
 * attributes for default Mfds are present.
 */
class MfdHelper2 {

  /*
   * For documentation:
   *
   * MFD default recombination rules
   *
   * Grid, Slab sources (with spatially varying rates)
   *
   * - a <Node> can only be of one type
   *
   * - will map to the default of that type (e.g. CA in 2008 had downweighted
   * M>6.5 in places that led to a node being either pure GR or INCR, but never
   * both)
   *
   * - only support multiple defaults of the same type
   *
   * Fault, Interface sources
   *
   * System
   *
   * - only supports a single SINGLE default MFD at this time
   *
   * How to manage MFD Type and Id mixing and matching?
   *
   * Other notes:
   *
   * Most source files will have a <settings> block; those that don't will not
   * trigger build() on a MfdHelper.Builder so most parsers preempt this
   * possibility be creating and empty Helper that is then ocverridden in most
   * cases.
   */

  // // mfd data instances
  // private final List<SingleData> singleDefaults;
  // private final List<GR_Data> grDefaults;
  // private final List<IncrData> incrDefaults;
  // private final List<TaperData> taperDefaults;
  //
  // private MfdHelper2(
  // List<SingleData> singleDefaults,
  // List<GR_Data> grDefaults,
  // List<IncrData> incrDefaults,
  // List<TaperData> taperDefaults) {
  //
  // this.singleDefaults = singleDefaults;
  // this.grDefaults = grDefaults;
  // this.incrDefaults = incrDefaults;
  // this.taperDefaults = taperDefaults;
  // }
  //
  // static Builder builder() {
  // return new Builder();
  // }
  //
  // List<SingleData> singleData(Attributes atts) {
  // if (singleDefaults.isEmpty()) {
  // return ImmutableList.of(new SingleData(atts));
  // }
  // List<SingleData> dataList = new ArrayList<>();
  // for (SingleData singleDefault : singleDefaults) {
  // dataList.add(new SingleData(atts, singleDefault));
  // }
  // Collections.sort(dataList);
  // return ImmutableList.copyOf(dataList);
  // }
  //
  // List<GR_Data> grData(Attributes atts) {
  // if (grDefaults.isEmpty()) {
  // return ImmutableList.of(new GR_Data(atts));
  // }
  // ImmutableList.Builder<GR_Data> builder = ImmutableList.builder();
  // for (GR_Data grDefault : grDefaults) {
  // builder.add(new GR_Data(atts, grDefault));
  // }
  // return builder.build();
  // }
  //
  // List<IncrData> incrementalData(Attributes atts) {
  // if (incrDefaults.isEmpty()) {
  // return ImmutableList.of(new IncrData(atts));
  // }
  // ImmutableList.Builder<IncrData> builder = ImmutableList.builder();
  // for (IncrData incrDefault : incrDefaults) {
  // builder.add(new IncrData(atts, incrDefault));
  // }
  // return builder.build();
  // }
  //
  // List<TaperData> taperData(Attributes atts) {
  // if (taperDefaults.isEmpty()) {
  // return ImmutableList.of(new TaperData(atts));
  // }
  // ImmutableList.Builder<TaperData> builder = ImmutableList.builder();
  // for (TaperData taperDefault : taperDefaults) {
  // builder.add(new TaperData(atts, taperDefault));
  // }
  // return builder.build();
  // }
  //
  // /*
  // * Used to by parsers to impose restrictions on the types of default MFDs
  // * allowed.
  // */
  // int typeCount(MfdType type) {
  // switch (type) {
  // case GR:
  // return grDefaults.size();
  // case INCR:
  // return incrDefaults.size();
  // case SINGLE:
  // return singleDefaults.size();
  // case GR_TAPER:
  // return taperDefaults.size();
  // default:
  // throw new IllegalArgumentException("Unknown MFD type: " + type);
  // }
  // }
  //
  // int size() {
  // int size = 0;
  // for (MfdType type : MfdType.values()) {
  // size += typeCount(type);
  // }
  // return size;
  // }
  //
  // /* Re-usable */
  // static final class Builder {
  //
  // private ImmutableList.Builder<SingleData> singleBuilder =
  // ImmutableList.builder();
  // private ImmutableList.Builder<GR_Data> grBuilder = ImmutableList.builder();
  // private ImmutableList.Builder<IncrData> incrBuilder =
  // ImmutableList.builder();
  // private ImmutableList.Builder<TaperData> taperBuilder =
  // ImmutableList.builder();
  //
  // /* Add a default MFD. */
  // Builder addDefault(Attributes atts) {
  // MfdType type = MfdType.valueOf(atts.getValue("type"));
  // switch (type) {
  // case GR:
  // grBuilder.add(new GR_Data(atts));
  // break;
  // case INCR:
  // incrBuilder.add(new IncrData(atts));
  // break;
  // case SINGLE:
  // singleBuilder.add(new SingleData(atts));
  // break;
  // case GR_TAPER:
  // taperBuilder.add(new TaperData(atts));
  // break;
  // default:
  // throw new IllegalArgumentException("Unknown MFD type: " + type);
  // }
  // return this;
  // }
  //
  // MfdHelper2 build() {
  // // MfdHelpers can be empty if no defaults
  // // defined for a SourceSet
  // return new MfdHelper2(
  // singleBuilder.build(),
  // grBuilder.build(),
  // incrBuilder.build(),
  // taperBuilder.build());
  // }
  // }

  private static final Gson GSON = new GsonBuilder().create();

  // static methods to build lists of mfds
  // if just single MFD is needed go straight to Mfd.builders
  // otherwise these methods can handle epistemic and aleatory and return lists
  // of mfds

  static void createSingle(JsonElement json) {

    // needs epi and aleatory config

    Single mfdData = GSON.fromJson(json, Single.class);

    // return pathways
    // - singleton tree with single m
    // - singleton tree with gaussian
    // - 3 branch tree with single m
    // - 3 branch tree with gaussian

    boolean epi = true;

  }

  static void createGutenbergRichter(JsonElement json) {

    // return pathways
    // - singleton tree with single m
    // - singleton tree with gaussian
    // - 3 branch tree with single m
    // - 3 branch tree with gaussian

    GutenbergRichter mfdData = GSON.fromJson(json, GutenbergRichter.class);
  }

  static void createTaperedGutenbergRichter(JsonElement json) {

    // ignores epi and alea

    GrTaper mfdData = GSON.fromJson(json, GrTaper.class);
  }

  static void createIncremental(JsonElement json) {

    // ignores epi and alea

    Incremental mfdData = GSON.fromJson(json, Incremental.class);
  }

  // TODO Comparable?

  // private List<IncrementalMfd> buildSingle(SingleData data) {
  //
  // List<IncrementalMfd> mfds = Lists.newArrayList();
  //
  // // total annual moment rate
  // double tmr = data.rate * Earthquakes.magToMoment(data.m);
  // // total annual event rate
  // double tcr = data.rate;
  //
  // // this was handled previously by GR_Data.hasMagExceptions()
  // // need to catch the single floaters that are less than 6.5
  // // note: unc.epiDeltas[0] may be null
  // double minUncertMag = data.m + (unc.hasEpistemic ? unc.epiDeltas[0] : 0.0);
  // boolean uncertAllowed = !((minUncertMag < unc.epiCutoff) && data.floats);
  //
  // // loop over epistemic uncertainties
  // if (unc.hasEpistemic && uncertAllowed) {
  // for (int i = 0; i < unc.epiCount; i++) {
  //
  // double epiMag = data.m + unc.epiDeltas[i];
  // double mfdWeight = data.weight * unc.epiWeights[i];
  //
  // if (unc.hasAleatory) {
  // IncrementalMfd mfd = (unc.moBalance)
  // ? Mfds.newGaussianMoBalancedMFD(
  // epiMag,
  // unc.aleaSigma,
  // unc.aleaCount,
  // mfdWeight * tmr,
  // data.floats)
  // : Mfds.newGaussianMFD(
  // epiMag,
  // unc.aleaSigma,
  // unc.aleaCount,
  // mfdWeight * tcr,
  // data.floats);
  // mfds.add(mfd);
  // log.finer(" MFD type: SINGLE [+epi +alea] " + epiBranch(i));
  // if (log.isLoggable(FINEST)) {
  // log.finest(mfd.getMetadataString());
  // }
  // } else {
  //
  // // single Mfds with epi uncertainty are moment balanced at
  // // the central/single magnitude of the distribution
  //
  // double moRate = tmr * mfdWeight;
  // IncrementalMfd mfd = Mfds.newSingleMoBalancedMFD(epiMag, moRate,
  // data.floats);
  // mfds.add(mfd);
  // log.finer(" MFD type: SINGLE [+epi -alea] " + epiBranch(i));
  // if (log.isLoggable(FINEST)) {
  // log.finest(mfd.getMetadataString());
  // }
  // }
  // }
  // } else {
  // if (unc.hasAleatory && uncertAllowed) {
  // IncrementalMfd mfd = (unc.moBalance)
  // ? Mfds.newGaussianMoBalancedMFD(
  // data.m,
  // unc.aleaSigma,
  // unc.aleaCount,
  // data.weight * tmr,
  // data.floats)
  // : Mfds.newGaussianMFD(
  // data.m,
  // unc.aleaSigma,
  // unc.aleaCount,
  // data.weight * tcr,
  // data.floats);
  // mfds.add(mfd);
  // log.finer(" MFD type: SINGLE [-epi +alea]");
  // if (log.isLoggable(FINEST)) {
  // log.finest(mfd.getMetadataString());
  // }
  // } else {
  // IncrementalMfd mfd =
  // Mfds.newSingleMFD(data.m, data.weight * data.rate, data.floats);
  // mfds.add(mfd);
  // log.finer(" MFD type: SINGLE [-epi -alea]");
  // if (log.isLoggable(FINEST)) {
  // log.finest(mfd.getMetadataString());
  // }
  // }
  // }
  // return mfds;
  // }

  private static String epiBranch(int i) {
    return (i == 0) ? "(M-epi)" : (i == 2) ? "(M+epi)" : "(M)";
  }

  static class MfdProperties {
    final Mfd.Type type;
    final Object properties;

    MfdProperties(Mfd.Type type, Object properties) {
      this.type = type;
      this.properties = properties;
    }
  }

  /* Single MFD data container. */
  static final class Single {
    double m;
    double rate;
  }

  /* Gutenberg–Richter MFD data container. */
  static final class GutenbergRichter {
    double a;
    double b;
    double dMag;
    double mMin;
    double mMax;
  }

  /* Tapered Gutenberg–Richter MFD data container. */
  static final class GrTaper {
    double a;
    double b;
    double cMag;
    double dMag;
    double mMin;
    double mMax;
  }

  /* Incremental MFD data container. */
  static final class Incremental {
    double[] mags;
    double[] rates;
  }

}
