package gov.usgs.earthquake.nshmp.eq.model;

import static com.google.common.base.Preconditions.checkNotNull;

import java.util.Map;
import java.util.Optional;

import gov.usgs.earthquake.nshmp.eq.model.MfdHelper2.MfdProperties;
import gov.usgs.earthquake.nshmp.mfd.Mfd;
import gov.usgs.earthquake.nshmp.tree.LogicTree;

/*
 * Rupture set container. A source wraps one or more rupture sets, each of which
 * has an MFD logic tree. The geometry associated with the set may be the same
 * or differnet than tht of other reupture sets associated with a source.
 */
class RuptureSet {

  final int id;
  final int[] sections;
  final LogicTree<MfdProperties> mfdTree;
  final Optional<LogicTree<Double>> rateTree;
  final MfdConfig mfdConfig;

  // TODO how to manage MFD tree, one or more branches, sometimes M1, M2, ...
  // in combination with an optional rate tree R1, R2, ... and possibly
  // layered with epistemic and aleatory variability

  // RuptureData

  RuptureSet(
      int id,
      int[] sections,
      LogicTree<MfdProperties> mfdTree,
      Optional<LogicTree<Double>> rateTree,
      MfdConfig mfdConfig) {

    this.id = id;
    this.sections = sections;
    this.mfdTree = mfdTree;
    this.rateTree = rateTree;
    this.mfdConfig = mfdConfig;
  }

  static Builder builder(ReferenceData data) {
    return new Builder(data);
  }

  static final class Builder {

    private ReferenceData data;

    private Integer id;
    private int[] sections;
    private LogicTree<MfdProperties> mfdTree;
    private LogicTree<Double> rateTree;

    private Builder(ReferenceData data) {
      this.data = data;
    }

    /* Set the rupture ID; may match a fault section ID. */
    Builder id(int id) {
      this.id = id;
      return this;
    }

    /* Set the section IDs; optional. */
    Builder sections(int[] sections) {
      this.sections = sections;
      return this;
    }

    /* Set the MFD tree using a mfdMap key. */
    Builder mfdTree(String key) {
      this.mfdTree = data.mfdMap.orElseThrow().get(key);
      return this;
    }

    /* Set the MFD tree. */
    Builder mfdTree(LogicTree<MfdProperties> mfdTree) {
      this.mfdTree = mfdTree;
      return this;
    }

    Builder rateTree(LogicTree<Double> rateTree) {
      this.rateTree = rateTree;
      return this;
    }

    // Builder ruptureData(Data )
    RuptureSet build() {
      checkNotNull(id, "Rupture geometry ID not set");
      if (sections == null) {
        sections = new int[] { id };
      }

      LogicTree<Mfd> mfdTree = buildMfdTree();
      return null;
    }

    private LogicTree<Mfd> buildMfdTree() {

      return null;
    }

    private LogicTree<Mfd> buildCollapsedMfdTree() {
      // collapse rate branches
      // mean hazard might as well use the total, n'est-ce pas?
      return null;
    }

    private Mfd buildTotalMfdTree() {
      // lazy?
      return null;
    }

  }

  /*
   * Stateful class that holds data (possibly) required by builder to create
   * final mfd-tree for rupture-set.
   */
  static final class ReferenceData {
    // SourceConfig sourceConfig; ?? probably
    MfdConfig mfdConfig;
    Optional<Map<String, LogicTree<MfdProperties>>> mfdMap;
    Optional<LogicTree<Double>> rateTree;
  }

  // static final class Geometry {
  //
  // }
}
