package gov.usgs.earthquake.nshmp.eq.fault;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertSame;
import static org.junit.jupiter.api.Assertions.assertThrows;

import org.junit.jupiter.api.Test;

import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.geo.LocationList;

@SuppressWarnings("javadoc")
public final class FaultsTests {

  @Test
  public final void testCheckFaultDipRakeStrike() {
    assertEquals(45.0, Faults.checkDip(45.0), 0.0);
    assertEquals(45.0, Faults.checkRake(45.0), 0.0);
    assertEquals(45.0, Faults.checkStrike(45.0), 0.0);

    assertThrows(IllegalArgumentException.class, () -> {
      Faults.checkDip(-0.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Faults.checkDip(90.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Faults.checkRake(-180.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Faults.checkRake(180.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Faults.checkStrike(-0.1);
    });

    assertThrows(IllegalArgumentException.class, () -> {
      Faults.checkStrike(360.0);
    });
  }

  @Test
  public final void testCheckTrace() {
    LocationList expect = LocationList.of(
        Location.create(0.0, 0.0),
        Location.create(1.0, 1.0));
    assertSame(expect, Faults.checkTrace(expect));

    assertThrows(IllegalArgumentException.class, () -> {
      Faults.checkTrace(LocationList.of(Location.create(0.0, 0.0)));
    });
  }

}
